package ${basePackage}.action.ajax.system;

import com.fasterxml.jackson.core.TreeNode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.multipart.MultipartFile;
import ${basePackage}.frame.excel.WSheet;
import ${basePackage}.frame.utils.LogUtil;
import ${basePackage}.frame.auth.LocalData;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.utils.MapperUtil;
import ${basePackage}.frame.base.BaseResponse;
import ${basePackage}.frame.utils.ResponseUtil;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.frame.excel.exception.ReadErrorException;
import ${basePackage}.frame.excel.exception.TemplateNotMatchException;
import ${basePackage}.module.system.ent.Tokens;
import ${basePackage}.module.system.mgr.TokensManager;
import ${basePackage}.module.system.req.TokensCreateRequest;
import ${basePackage}.module.system.req.TokensDeleteRequest;
import ${basePackage}.module.system.req.TokensFindRequest;
import ${basePackage}.module.system.req.TokensGetRequest;
import ${basePackage}.module.system.req.TokensUpdateRequest;
import ${basePackage}.module.system.rsp.TokensCreateResponse;
import ${basePackage}.module.system.rsp.TokensDeleteResponse;
import ${basePackage}.module.system.rsp.TokensFindResponse;
import ${basePackage}.module.system.rsp.TokensGetResponse;
import ${basePackage}.module.system.rsp.TokensUpdateResponse;

import java.io.IOException;
import java.util.List;

public class TokensAjax{

    @Autowired
    private TokensManager tokensManager;

    public TokensCreateResponse create(TokensCreateRequest request) {
        return tokensManager.create(request, LocalData.getToken());
    }

    public TokensDeleteResponse delete(TokensDeleteRequest request) {
        return tokensManager.delete(request, LocalData.getToken());
    }

    public TokensUpdateResponse update(TokensUpdateRequest request) {
        return tokensManager.update(request, LocalData.getToken());
    }

    public TokensFindResponse find(TokensFindRequest request) {
        return tokensManager.find(request, LocalData.getToken());
    }

    public TokensGetResponse get(TokensGetRequest request) {
        return tokensManager.get(request, LocalData.getToken());
    }

    public Object template(){
        return ResponseUtil.apply(new WSheet<>(Tokens.class));
    }

    public Object exports(TreeNode jsonParam) {
        TokensFindRequest request = MapperUtil.toJava(jsonParam, TokensFindRequest.class);
        TokensFindResponse response = tokensManager.find(request, LocalData.getToken());
        if (response.hasError()) {
            return response;
        } else if (response.getTotalCount() == 0) {
            response.addError(ErrorType.BUSINESS_ERROR, "导出数据为空");
            return response;
        }
        return ResponseUtil.apply(new WSheet<>(response.getResult()));
    }

    public Object imports(MultipartFile file) {
        BaseResponse baseResponse = new BaseResponse();
        try {
            WSheet<Tokens> sheet = new WSheet<>(file.getBytes(), Tokens.class, new WSheet.Validator<Tokens>() {
                @Override
                public List<String> validate(Tokens o) {
                    TokensCreateRequest request = MapperUtil.map(o, TokensCreateRequest.class);
                    return ValidationUtil.validate(request);
                }
            });

            if (sheet.hasError()) {
                return ResponseUtil.apply(sheet.getBytes(), sheet.getName() + "-检查.xlsx");
            } else {
                return baseResponse;
            }
        } catch (IOException e) {
            e.printStackTrace();
            LogUtil.dumpException(e);
            baseResponse.addError(ErrorType.BUSINESS_ERROR, "上传文件出错");
        } catch (TemplateNotMatchException | ReadErrorException e) {
            baseResponse.addError(ErrorType.BUSINESS_ERROR, e.getMessage());
        }
        return baseResponse;
    }
}
